@@ -27,8 +27,8 @@
 /*
  * Define the SIP version number.
  */
-#define SIP_VERSION         0x040c01
-#define SIP_VERSION_STR     "4.12.1"
+#define SIP_VERSION         0x040c04
+#define SIP_VERSION_STR     "4.12.4"
 
 
 #ifdef TRUE
@@ -607,10 +607,10 @@ typedef enum {
 /* A software license. */
 
 typedef struct {
-    char *type;                         /* The license type. */
-    char *licensee;                     /* The licensee. */
-    char *timestamp;                    /* The timestamp. */
-    char *sig;                          /* The signature. */
+    const char *type;                   /* The license type. */
+    const char *licensee;               /* The licensee. */
+    const char *timestamp;              /* The timestamp. */
+    const char *sig;                    /* The signature. */
 } licenseDef;
 
 
@@ -651,10 +651,17 @@ typedef struct _codeBlock {
     char *frag;                         /* The code itself. */
     const char *filename;               /* The original file. */
     int linenr;                         /* The line in the file. */
-    struct _codeBlock *next;            /* Next in the list. */
 } codeBlock;
 
 
+/* A list of literal code blocks. */
+
+typedef struct _codeBlockList {
+    codeBlock *block;                   /* The code block. */
+    struct _codeBlockList *next;        /* The next in the list. */
+} codeBlockList;
+
+
 /* The arguments to a throw specifier. */
 
 typedef struct _throwArgs {
@@ -672,7 +679,7 @@ typedef struct _exceptionDef {
     struct _classDef *cd;               /* The exception class. */
     char *bibase;                       /* The builtin base exception. */
     struct _exceptionDef *base;         /* The defined base exception. */
-    codeBlock *raisecode;               /* Raise exception code. */
+    codeBlockList *raisecode;           /* Raise exception code. */
     struct _exceptionDef *next;         /* The next in the list. */
 } exceptionDef;
 
@@ -762,15 +769,15 @@ typedef struct _moduleDef {
     nameDef *defmetatype;               /* The optional default meta-type. */
     nameDef *defsupertype;              /* The optional default super-type. */
     struct _exceptionDef *defexception; /* The default exception. */
-    codeBlock *hdrcode;                 /* Header code. */
-    codeBlock *cppcode;                 /* Global C++ code. */
-    codeBlock *copying;                 /* Software license. */
-    codeBlock *preinitcode;             /* Pre-initialisation code. */
-    codeBlock *initcode;                /* Initialisation code. */
-    codeBlock *postinitcode;            /* Post-initialisation code. */
-    codeBlock *unitcode;                /* Compilation unit code. */
-    codeBlock *unitpostinccode;         /* Compilation unit post-include code. */
-    codeBlock *docstring;               /* The docstring. */
+    codeBlockList *hdrcode;             /* Header code. */
+    codeBlockList *cppcode;             /* Global C++ code. */
+    codeBlockList *copying;             /* Software license. */
+    codeBlockList *preinitcode;         /* Pre-initialisation code. */
+    codeBlockList *initcode;            /* Initialisation code. */
+    codeBlockList *postinitcode;        /* Post-initialisation code. */
+    codeBlockList *unitcode;            /* Compilation unit code. */
+    codeBlockList *unitpostinccode;     /* Compilation unit post-include code. */
+    codeBlockList *docstring;           /* The docstring. */
     int parts;                          /* The number of parts generated. */
     const char *file;                   /* The filename. */
     qualDef *qualifiers;                /* The list of qualifiers. */
@@ -811,7 +818,7 @@ typedef struct _ifaceFileDef {
     int ifacenr;                        /* The index into the types table. */
     scopedNameDef *fqcname;             /* The fully qualified C++ name. */
     moduleDef *module;                  /* The owning module. */
-    codeBlock *hdrcode;                 /* Header code. */
+    codeBlockList *hdrcode;             /* Header code. */
     struct _ifaceFileList *used;        /* Interface files used. */
     struct _ifaceFileDef *next;         /* Next in the list. */
 } ifaceFileDef;
@@ -836,9 +843,9 @@ typedef struct _mappedTypeDef {
     ifaceFileDef *iff;                  /* The interface file. */
     struct _memberDef *members;         /* The static member functions. */
     struct _overDef *overs;             /* The static overloads. */
-    codeBlock *typecode;                /* Type code. */
-    codeBlock *convfromcode;            /* Convert from C++ code. */
-    codeBlock *convtocode;              /* Convert to C++ code. */
+    codeBlockList *typecode;            /* Type code. */
+    codeBlockList *convfromcode;        /* Convert from C++ code. */
+    codeBlockList *convtocode;          /* Convert to C++ code. */
     struct _mappedTypeDef *next;        /* Next in the list. */
 } mappedTypeDef;
 
@@ -876,7 +883,7 @@ typedef struct _virtHandlerDef {
     signatureDef *pysig;                /* The Python signature. */
     signatureDef *cppsig;               /* The C++ signature. */
     struct _moduleDef *module;          /* The defining module. */
-    codeBlock *virtcode;                /* Virtual handler code. */
+    codeBlockList *virtcode;            /* Virtual handler code. */
     struct _virtHandlerDef *next;       /* Next in the list. */
 } virtHandlerDef;
 
@@ -902,9 +909,9 @@ typedef struct _varDef {
     moduleDef *module;                  /* The owning module. */
     int varflags;                       /* The variable flags. */
     argDef type;                        /* The actual type. */
-    codeBlock *accessfunc;              /* The access function. */
-    codeBlock *getcode;                 /* The get code. */
-    codeBlock *setcode;                 /* The set code. */
+    codeBlockList *accessfunc;          /* The access function. */
+    codeBlockList *getcode;             /* The get code. */
+    codeBlockList *setcode;             /* The set code. */
     struct _varDef *next;               /* Next in the list. */
 } varDef;
 
@@ -915,7 +922,7 @@ typedef struct _propertyDef {
     nameDef *name;                      /* The property name. */
     const char *get;                    /* The name of the getter method. */
     const char *set;                    /* The name of the setter method. */
-    codeBlock *docstring;               /* The docstring. */
+    codeBlockList *docstring;           /* The docstring. */
     struct _propertyDef *next;          /* Next in the list. */
 } propertyDef;
 
@@ -931,7 +938,7 @@ typedef struct _overDef {
     signatureDef pysig;                 /* The Python signature. */
     signatureDef *cppsig;               /* The C++ signature. */
     throwArgs *exceptions;              /* The exceptions. */
-    codeBlock *methodcode;              /* Method code. */
+    codeBlockList *methodcode;          /* Method code. */
     virtHandlerDef *virthandler;        /* The virtual handler. */
     char *prehook;                      /* The pre-hook name. */
     char *posthook;                     /* The post-hook name. */
@@ -948,7 +955,7 @@ typedef struct _ctorDef {
     signatureDef pysig;                 /* The Python signature. */
     signatureDef *cppsig;               /* The C++ signature, NULL if /NoDerived/. */
     throwArgs *exceptions;              /* The exceptions. */
-    codeBlock *methodcode;              /* Method code. */
+    codeBlockList *methodcode;          /* Method code. */
     char *prehook;                      /* The pre-hook name. */
     char *posthook;                     /* The post-hook name. */
     struct _ctorDef *next;              /* Next in the list. */
@@ -994,7 +1001,7 @@ typedef struct _memberDef {
     int membernr;                       /* The index in the method table. */
     slotType slot;                      /* The slot type. */
     moduleDef *module;                  /* The owning module. */
-    codeBlock *docstring;               /* The overloads docstrings. */
+    codeBlockList *docstring;           /* The overloads docstrings. */
     struct _memberDef *next;            /* Next in the list. */
 } memberDef;
 
@@ -1050,28 +1057,28 @@ typedef struct _classDef {
     templateDef *td;                    /* The instantiated template. */
     ctorDef *ctors;                     /* The constructors. */
     ctorDef *defctor;                   /* The default ctor. */
-    codeBlock *dealloccode;             /* Handwritten dealloc code. */
-    codeBlock *dtorcode;                /* Handwritten dtor code. */
+    codeBlockList *dealloccode;         /* Handwritten dealloc code. */
+    codeBlockList *dtorcode;            /* Handwritten dtor code. */
     throwArgs *dtorexceptions;          /* The dtor exceptions. */
     memberDef *members;                 /* The member functions. */
     overDef *overs;                     /* The overloads. */
     argList *casts;                     /* The operator casts. */
     virtOverDef *vmembers;              /* The virtual members. */
     visibleList *visible;               /* The visible members. */
-    codeBlock *cppcode;                 /* Class C++ code. */
-    codeBlock *convtosubcode;           /* Convert to sub C++ code. */
+    codeBlockList *cppcode;             /* Class C++ code. */
+    codeBlockList *convtosubcode;       /* Convert to sub C++ code. */
     struct _classDef *subbase;          /* Sub-class base class. */
-    codeBlock *docstring;               /* Ctor docstrings. */
-    codeBlock *convtocode;              /* Convert to C++ code. */
-    codeBlock *travcode;                /* Traverse code. */
-    codeBlock *clearcode;               /* Clear code. */
-    codeBlock *getbufcode;              /* Get buffer code (Python v3). */
-    codeBlock *releasebufcode;          /* Release buffer code (Python v3). */
-    codeBlock *readbufcode;             /* Read buffer code (Python v2). */
-    codeBlock *writebufcode;            /* Write buffer code (Python v2). */
-    codeBlock *segcountcode;            /* Segment count code (Python v2). */
-    codeBlock *charbufcode;             /* Character buffer code (Python v2). */
-    codeBlock *picklecode;              /* Pickle code. */
+    codeBlockList *docstring;           /* Class and ctor docstrings. */
+    codeBlockList *convtocode;          /* Convert to C++ code. */
+    codeBlockList *travcode;            /* Traverse code. */
+    codeBlockList *clearcode;           /* Clear code. */
+    codeBlockList *getbufcode;          /* Get buffer code (Python v3). */
+    codeBlockList *releasebufcode;      /* Release buffer code (Python v3). */
+    codeBlockList *readbufcode;         /* Read buffer code (Python v2). */
+    codeBlockList *writebufcode;        /* Write buffer code (Python v2). */
+    codeBlockList *segcountcode;        /* Segment count code (Python v2). */
+    codeBlockList *charbufcode;         /* Character buffer code (Python v2). */
+    codeBlockList *picklecode;          /* Pickle code. */
     propertyDef *properties;            /* The properties. */
     struct _classDef *next;             /* Next in the list. */
 } classDef;
@@ -1136,8 +1143,8 @@ typedef struct {
     enumDef *enums;                     /* List of enums. */
     varDef *vars;                       /* List of variables. */
     typedefDef *typedefs;               /* List of typedefs. */
-    codeBlock *exphdrcode;              /* Exported header code. */
-    codeBlock *docs;                    /* Documentation. */
+    codeBlockList *exphdrcode;          /* Exported header code. */
+    codeBlockList *docs;                /* Documentation. */
     int sigslots;                       /* Set if signals or slots are used. */
     int genc;                           /* Set if we are generating C code. */
     struct _stringList *plugins;        /* The list of plugins. */
@@ -1199,7 +1206,7 @@ scopedNameDef *copyScopedName(scopedNameDef *);
 void appendScopedName(scopedNameDef **,scopedNameDef *);
 void freeScopedName(scopedNameDef *);
 void appendToClassList(classList **,classDef *);
-void appendCodeBlock(codeBlock **headp, codeBlock *new);
+void appendCodeBlockList(codeBlockList **headp, codeBlockList *cbl);
 void prcode(FILE *fp, const char *fmt, ...);
 void prOverloadName(FILE *fp, overDef *od);
 void prOverloadDecl(FILE *fp, ifaceFileDef *scope, overDef *od, int defval);
@@ -1217,7 +1224,8 @@ int isRichCompareSlot(memberDef *md);
 mappedTypeDef *allocMappedType(sipSpec *pt, argDef *type);
 void appendString(stringList **headp, const char *s);
 void appendTypeStrings(scopedNameDef *ename, signatureDef *patt, signatureDef *src, signatureDef *known, scopedNameDef **names, scopedNameDef **values);
-codeBlock *templateCode(sipSpec *pt, ifaceFileList **used, codeBlock *ocb, scopedNameDef *names, scopedNameDef *values);
+codeBlockList *templateCode(sipSpec *pt, ifaceFileList **used,
+        codeBlockList *ocbl, scopedNameDef *names, scopedNameDef *values);
 ifaceFileDef *findIfaceFile(sipSpec *pt, moduleDef *mod,
         scopedNameDef *fqname, ifaceFileType iftype,
         apiVersionRangeDef *api_range, argDef *ad);
